package com.ruoyi.pt.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.pt.entity.CaseInfo;
import com.ruoyi.pt.entity.CasesImport;
import com.ruoyi.pt.entity.Events;
import com.ruoyi.pt.entity.dto.AEventChangeIsReport;
import com.ruoyi.pt.entity.dto.AMassEventsRequest;
import com.ruoyi.pt.entity.dto.ASimilarRequest;
import com.ruoyi.pt.entity.response.ANew100PageResponse;
import com.ruoyi.pt.entity.response.FiveEventCountResponse;
import com.ruoyi.pt.service.AEventsService;
import com.ruoyi.pt.service.EventsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

/**
 * 事件控制层
 * @author du
 * @since 2024/9/2 10:20
 */
@Api(tags = "事件")
@RestController
@RequestMapping("/earlyWarningAudit/massEvents")
public class AEventsController extends BaseController {

    @Resource
    private AEventsService aEventsService;

    @Resource
    private EventsService eventsService;

    @Resource
    private SysLoginService sysLoginService;
    /**
     * 登录获取token
     */
    @PostMapping("/login")
    @ApiOperation(value = "登录获取token")
    public AjaxResult login(@RequestBody LoginBody loginBody) {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = sysLoginService.login1(loginBody.getUsername(), loginBody.getPassword());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * app登录获取token
     */
    @PostMapping("/appLogin")
    @ApiOperation(value = "app登录获取token")
    public AjaxResult appLogin(@RequestBody LoginBody loginBody) {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = sysLoginService.login2(loginBody.getUsername(), loginBody.getPassword());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }


//    /**
//     * 取最新一条事件（预警钟）
//     */
//    @GetMapping("/limitOneMsg")
//    @ApiOperation(value = "取最新一条事件（预警钟）", response = Events.class)
//    public AjaxResult limitOneMsg() {
//        return success(aEventsService.limitOneMsg());
//    }

    /**
     * 根据100条最新事件查询对应工单
     */
    @GetMapping("/new100Page")
    @ApiOperation(value = "根据100条最新事件查询对应工单", response = ANew100PageResponse.class)
    public AjaxResult new100Page() {
        return success(aEventsService.new100Page());
    }


    /**
     * 获取日志错误信息
     */
    @GetMapping("/getErrorMsg")
    @ApiOperation(value = "获取日志错误信息",response = Events.class)
    public AjaxResult getErrorMsg(HttpServletRequest request) {
        String header1 = request.getHeader("clientId");
        String header2 = request.getHeader("clientSecret");
        if(header1==null||header2==null){
           return AjaxResult.error(401,"请求访问：/earlyWarningAudit/massEvents/getErrorMsg，认证失败，无法访问系统资源");
        }else {
            if("3YSE6O8fRACPPRVCqAlxgkhzMG+tasXXYgdDsiWPi/U=".equals(header1)
                    && "kdoiIv5Wn2VvPG0sSGNF9Q4SVvu3CoBy+m6e8skq3AY=".equals(header2)){
                return success(aEventsService.getErrorMsg());
            }else {
                return AjaxResult.error(401,"请求访问：/earlyWarningAudit/massEvents/getErrorMsg，认证失败，无法访问系统资源");
            }
        }
    }

    /**
     * 分页条件查询事件数据
     *
     * @param page   分页条件
     * @param events 查询条件
     * @return 所有数据
     */
    @GetMapping("/colonyPage")
    @ApiOperation(value = "分页条件查询事件数据", response = Events.class)
    public AjaxResult colonyPage(Page<Events> page, AMassEventsRequest events) {
        return success(aEventsService.page(page,events));
    }

    /**
     * 根据查询条件查询事件待审核个数
     *
     * @param events 查询条件
     * @return 所有数据
     */
    @GetMapping("/eventPageCount")
    @ApiOperation(value = "根据查询条件查询事件待审核个数")
    public AjaxResult eventPageCount(AMassEventsRequest events) {
        return success(aEventsService.eventPageCount(events));
    }

    /**
     * 查询五种事件待审核个数
     */
    @GetMapping("/fiveEventCount")
    @ApiOperation(value = "查询五种事件待审核个数",response = FiveEventCountResponse.class)
    public AjaxResult fiveEventCount() {
        return success(aEventsService.fiveEventCount());
    }

//    /**
//     * 修改事件标题
//     */
//    @ApiOperation(value = "修改事件标题")
//    @PostMapping("/edit")
//    public AjaxResult edit(@RequestBody Events events) {
//        Events byId = aEventsService.getByInnerEventId(events.getInnerEventId());
//        events.setTitleBefore(byId.getTitle());
//        return toAjax(aEventsService.edit(events));
//    }

    /**
     * 根据事件的innerEventId查询事件详情
     */
    @GetMapping("/getByInnerEventId/{innerEventId}")
    @ApiOperation(value = "根据事件的innerEventId查询事件详情", response = Events.class)
    public AjaxResult getByInnerEventId( @PathVariable String innerEventId) {
        return success(aEventsService.getByInnerEventId(innerEventId));
    }

    /**
     * 根据事件的innerEventId分页查询审核工单列表
     */
    @GetMapping("/auditList/{innerEventId}")
    @ApiOperation(value = "根据事件的innerEventId分页查询审核工单列表", response = CasesImport.class)
    public AjaxResult auditList(Page<CasesImport> page, @PathVariable String innerEventId) {
        return success(aEventsService.auditList(page,innerEventId));
    }

    /**
     * 查询工单详情
     */
    @ApiOperation(value = "查询工单详情")
    @GetMapping(value = "/{caseSerial}")
    public AjaxResult getById(@PathVariable("caseSerial") String id) {
        return success(aEventsService.getById(id));
    }

    /**
     * 事件提级或退回
     */
    @ApiOperation(value = "事件提级或退回")
    @PostMapping("/eventChangeIsReport")
    public AjaxResult eventChangeIsReport(@RequestBody @Valid AEventChangeIsReport events) {
//        if(events.getCaseSerial().isEmpty()){
//            throw new ServiceException("请选择工单上报!");
//        }
        aEventsService.eventChangeIsReport(events);
        return success();
    }
}
