package com.ruoyi.pt.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.pt.entity.CaseInfo;
import com.ruoyi.pt.service.ICaseInfoService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 服务工单受理Controller
 *
 * @author ruoyi
 * @date 2023-11-27
 */
@Api(tags = "服务工单受理")
@RestController
@RequestMapping("/pt/info")
public class CaseInfoController extends BaseController {
    @Autowired
    private ICaseInfoService caseInfoService;

    /**
     * 查询服务工单受理列表
     */
    @ApiOperation(value = "查询服务工单受理列表", response = CaseInfo.class)
    @GetMapping("/list")
    public TableDataInfo list(CaseInfo caseInfo) {
        startPage();
        List<CaseInfo> list = caseInfoService.selectCaseInfoList(caseInfo);
        return getDataTable(list);
    }

    /**
     * 导出服务工单受理列表
     */
    @ApiOperation(value = "导出服务工单受理列表")
    @PostMapping("/export")
    public void export(HttpServletResponse response, CaseInfo caseInfo) {
        List<CaseInfo> list = caseInfoService.selectCaseInfoList(caseInfo);
        ExcelUtil<CaseInfo> util = new ExcelUtil<CaseInfo>(CaseInfo. class);
        util.exportExcel(response, list, "服务工单受理数据");
    }

    /**
     * 获取服务工单受理详细信息
     */
    @ApiOperation(value = " 获取服务工单受理详细信息")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(caseInfoService.selectCaseInfoById(id));
    }

    /**
     * 新增服务工单受理
     */
    @ApiOperation(value = " 新增服务工单受理")
    @PostMapping
    public AjaxResult add(@RequestBody CaseInfo caseInfo) {
        return toAjax(caseInfoService.insertCaseInfo(caseInfo));
    }

    /**
     * 修改服务工单受理
     */
    @ApiOperation(value = " 修改服务工单受理")
    @PutMapping
    public AjaxResult edit(@RequestBody CaseInfo caseInfo) {
        return toAjax(caseInfoService.updateCaseInfo(caseInfo));
    }

    /**
     * 删除服务工单受理
     */
    @ApiOperation(value = " 删除服务工单受理")
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(caseInfoService.deleteCaseInfoByIds(ids));
    }
}
