package com.ruoyi.pt.entity.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

/**
 * 远程调用事件推送接口请求类
 *
 * @author wu
 * @since 2023/12/15 16:35
 */
@Data
public class RemoteCallsDTO implements Serializable {

    /**
     * 事件提供方的对事件的唯一标识
     */
    private String innerEventId;

    /**
     * 事件消息所属区域15位编码
     * 固定值：320505000000000
     */
    private String areaCode = "320505000000000";

    /**
     * 件消息所属场景编码：该场
     * 景编码由各提供⽅向指挥中
     * ⼼平台申请 固定值: sjsffx
     */
    private String scence = "sjsffx";

    /**
     * 事件消息标题
     */
    private String title;

    /**
     * 事件消息内容描述
     */
    private String content;

    /**
     * 事件发⽣时间，格式为：yyyy-MM-dd HH:mm:ss
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date eventTimeDate;

    private String lastWarnTime;

    private String eventTime;

    private String firstWarnTime;

    /**
     * 事件类型编码
     */
    private String msgType;

    /**
     * 事件类型名称
     */
    private String msgTypeName;

    /**
     * 事件执⾏类型 1：流转 2：
     * 展示 3：提级
     * 【暂时默认都传展示】
     */
    private Integer executeType = 2;

    /**
     * 事件是否超时: 【超期办
     * 结】传【1】
     * 其他传【0】
     */
    private Integer remindStatus = 0;

    /**
     * 1、⼀般2、提级3、疑难4、
     * 多跨5、逾期6、推诿7、督
     * 办，多个逗号分隔
     * 默认为⼀般
     */
    private String eventLabel = "1";

    /**
     * 案件类型编码
     */
    private String scenceType;

    /**
     * 案件类型名称
     */
    private String scenceTypename;

    /**
     * 预警关联的事件单号(案件单号)
     */
    private String relationNums;

    /**
     * 预警因素
     */
    private String warnFactor;

    /**
     * 第⼀次预警时间 格式为：yyyy-MM-dd HH:mm:ss
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date firstWarnTimeDate;

    /**
     * 事件上报人
     */
    private String eventCreator;

    /**
     * 事件上报人联系方式
     */
    private String eventCreatorTel;

    /**
     * 事件发生地名
     */
    private String eventPlaceName;

    /**
     * 经纬度
     */
    private String eventCoordinate;
}
