package com.ruoyi.pt.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.pt.entity.CasesImport;
import com.ruoyi.pt.entity.Events;
import com.ruoyi.pt.entity.dto.AEventChangeIsReport;
import com.ruoyi.pt.entity.dto.AMassEventsRequest;
import com.ruoyi.pt.entity.dto.RemoteCallsDTO;
import com.ruoyi.pt.entity.response.ANew100PageResponse;
import com.ruoyi.pt.entity.response.FiveEventCountResponse;
import com.ruoyi.pt.mapper.AEventsMapper;
import com.ruoyi.pt.service.AEventsService;
import com.ruoyi.pt.service.CasesImportService;
import com.ruoyi.pt.service.RemoteCallsService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 群体事件业务处理层
 *
 * @author du
 * @since 2024/9/2 10:24
 */
@Service
public class AEventsServiceImpl implements AEventsService {

    public static final Logger log = LoggerFactory.getLogger(AEventsServiceImpl.class);
    public static final ArrayList<Object> OBJECTS = new ArrayList<>();

    @Resource
    private RedisCache redisCache;

    @Resource
    private AEventsMapper aEventsMapper;

    @Resource
    private CasesImportService casesImportService;

    @Resource
    private RemoteCallsService remoteCallsService;

    @Value("${remote.url}")
    private String url;

    /**
     * 分页条件查询事件表
     */
    @Override
    public Page<Events> page(Page<Events> page, AMassEventsRequest events) {
        Page<Events> page1 = aEventsMapper.page(page, events);
        for (Events ev : page1.getRecords()) {
            String str = ev.getRelationNums().replaceAll("[\\[\\]']", "");
            List<String> list = Arrays.asList(str.split(",\\s*"));
            if (!list.isEmpty()) {
                CasesImport byId = aEventsMapper.getById(list.get(0));
                ev.setAddress(byId.getCaseAddress());
            }
        }

        return page1;
    }


    /**
     * 分页条件查询审核列表
     */
    @Override
    public Page<CasesImport> auditList(Page<CasesImport> page, String innerEventId) {
        //获取该事件详细信息
        Events ev = aEventsMapper.getByInnerEventId(innerEventId);
        String str = ev.getRelationNums().replaceAll("[\\[\\]']", "");
        return aEventsMapper.auditList(page, Arrays.asList(str.split(",\\s*")));
    }

    /**
     * 查询工单详情
     */
    @Override
    public CasesImport getById(String id) {
        return aEventsMapper.getById(id);
    }

    /**
     * 修改事件标题
     */
    @Override
    public int edit(Events events) {
        return aEventsMapper.edit(events);
    }


    /**
     * 事件提级或退回
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void eventChangeIsReport(AEventChangeIsReport events) {
        events.setAuditTime(DateUtil.date());
        //获取当前innerEventId对应的事件
        Events be = aEventsMapper.getByInnerEventId(events.getInnerEventId());
        String str = be.getRelationNums().replaceAll("[\\[\\]']", "");
        //当前innerEventId对应的事件对应的工单
        List<String> sp = Arrays.asList(str.split(",\\s*"));
        List<String> ol = new ArrayList<>(sp); // 可变集合
        //如果工单编号数组不为0,则将其工单进行标记
        if (events.getCaseSerial() != null && !events.getCaseSerial().isEmpty()) {
            aEventsMapper.updateCasesImport(events.getCaseSerial(), 1, DateUtil.now());
            ol.removeAll(events.getCaseSerial());
        } else {
            //如果不更改 都更改审批时间
            aEventsMapper.updateCasesImport(ol, null, DateUtil.now());
        }
        aEventsMapper.updateEvent(events);
        if (events.getIsReport() == 1) {
            Events a1 = aEventsMapper.getByInnerEventId(events.getInnerEventId());
            StringBuilder sb = new StringBuilder().append("[");
            int count = 1;
            for (String x : ol) {
                sb.append("'");
                sb.append(x);
                sb.append("'");
                if (count != events.getCaseSerial().size()) {
                    sb.append(",");
                }
                count++;
            }
            sb.append("]");
            a1.setRelationNums(sb.toString());
            String substring = a1.getInnerEventId().substring(0, 6);
            a1.setInnerEventId(substring + IdUtil.randomUUID());

            if (!ol.isEmpty()) {
                aEventsMapper.addAudit(a1);
                // 重新上报驾驶舱 todo al.getRelationNums() 需要符合的工单id
                RemoteCallsDTO dto = BeanUtil.copyProperties(a1, RemoteCallsDTO.class);
                dto.setContent(dto.getTitle());
                // 创建SimpleDateFormat对象，指定日期格式为yyyy-MM-dd HH:mm:ss
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                // 使用SimpleDateFormat对象将Date对象格式化为字符串
                String firstWarnTime = sdf.format(dto.getFirstWarnTimeDate());
                String eventTime = sdf.format(dto.getEventTimeDate());
                dto.setFirstWarnTime(firstWarnTime);
                dto.setEventTime(firstWarnTime);
                dto.setLastWarnTime(eventTime);
                // 获取relationNums转成数组
//                JSONArray jsonArray = JSONUtil.parseArray(dto.getRelationNums());
//                List<String> list = jsonArray.toList(String.class);
                QueryWrapper<CasesImport> wrapper = new QueryWrapper<>();
                wrapper.eq("CASE_SERIAL", ol.get(0));
                CasesImport casesImport = casesImportService.getOne(wrapper);
                if (BeanUtil.isNotEmpty(casesImport)) {
                    // 如果是紧急事件content使用原始工单的content
                    if ("3".equals(dto.getScenceType())) {
                        dto.setContent(casesImport.getCaseContent());
                    } else {
                        dto.setContent(dto.getTitle());
                    }
                    // 如果类型是同人事件，上报⼈姓名和电话不取原始工单，直接从预警因素字段取，顿号分隔。
                    if ("2".equals(dto.getScenceType())) {
                        String warnFactor = dto.getWarnFactor();
                        String[] split = warnFactor.split("、");
                        if (split.length == 2) {
                            dto.setEventCreatorTel(split[0]);
                            dto.setEventCreator(split[1]);
                        } else {
                            dto.setEventCreatorTel(split[0]);
                        }
                    } else {
                        dto.setEventCreator(casesImport.getCreatorName());
                        dto.setEventCreatorTel(casesImport.getCreatorTel());
                    }
                    dto.setEventPlaceName(casesImport.getCaseAddress());
                    dto.setEventCoordinate(casesImport.getCaseLnglat());
                    String jsonStr = JSONUtil.toJsonStr(dto);
                    String accessToken = remoteCallsService.getAccessToken();
                    HttpRequest.post(url + "/gateway/event/event/eventData/imports")
                            .header("Authorization", "Bearer " + accessToken)
                            .body(jsonStr).execute().body();
                    log.info("==========重新推送驾驶舱完成=========");
                }
            }
        }
    }

    /**
     * 根据100条最新事件查询对应工单
     */
    @Override
    public List<ANew100PageResponse> new100Page() {
        //查询最新100条事件
        Page<Events> page1 = new Page<>();
        page1.setCurrent(1);
        page1.setSize(100);
        Page<Events> page = aEventsMapper.page(page1, new AMassEventsRequest());
        Map<String, String> msg = new HashMap<>();
        List<String> a1 = new ArrayList<>();
        page.getRecords().forEach(x -> {
            String[] split = x.getRelationNums().replaceAll("[\\[\\]']", "").split(",\\s*");
            if (split.length > 0) {
                a1.add(split[0]);
                msg.put(split[0], x.getInnerEventId());
            }
        });
        List<ANew100PageResponse> new100PageList = aEventsMapper.new100Page(a1);
        new100PageList.forEach(y -> {
            String s = msg.get(y.getCaseSerial());
            page.getRecords().forEach(x -> {
                if (s != null && s.equals(x.getInnerEventId())) {
                    y.setNums(x.getNums());
                    y.setMsgType(x.getMsgType());
                    y.setInnerEventId(x.getInnerEventId());
                    y.setRelationNums(x.getRelationNums());
                    y.setAuditTime(x.getAuditTime());
                    y.setEventTimeDate(x.getEventTimeDate());
                    y.setFirstWarnTimeDate(x.getFirstWarnTimeDate());
                    y.setIsReport(x.getIsReport());
                    y.setMsgTypeName(x.getMsgTypeName());
                    y.setScenceName(x.getScenceName());
                    y.setScenceType(x.getScenceType());
                    y.setScenceTypeName(x.getScenceTypeName());
                    y.setState(x.getState());
                    y.setTitle(x.getTitle());
                    y.setTitleBefore(x.getTitleBefore());
                    y.setWarnFactor(x.getWarnFactor());
                }
            });
        });
        return new100PageList;
    }

    /**
     * 查询事件详情
     */
    @Override
    public Events getByInnerEventId(String innerEventId) {
        return aEventsMapper.getByInnerEventId(innerEventId);
    }

    /**
     * 取最新一条事件
     */
    @Override
    public List<Events> limitOneMsg() {
        return aEventsMapper.limitOneMsg();
    }

    /**
     * 根据查询条件查询事件待审核个数
     *
     * @param events 查询条件
     * @return 所有数据
     */
    @Override
    public Integer eventPageCount(AMassEventsRequest events) {
        return aEventsMapper.eventPageCount(events);
    }

    /**
     * 查询五种事件待审核个数
     */
    @Override
    public List<FiveEventCountResponse> fiveEventCount() {
        return aEventsMapper.fiveEventCount();
    }

    /**
     * 获取日志错误信息
     */
    @Override
    public Object getErrorMsg() {
        if (redisCache.hasKey("isNo")) {
            return "算法运行异常";
        } else {
            return aEventsMapper.limitOneMsg();
        }
    }
}
