package com.ruoyi.pt.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpException;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.pt.entity.CasesImport;
import com.ruoyi.pt.entity.Events;
import com.ruoyi.pt.entity.dto.RemoteCallsDTO;
import com.ruoyi.pt.entity.response.PassEventRequest;
import com.ruoyi.pt.entity.response.RemoteCallsResponse;
import com.ruoyi.pt.entity.response.RemoteEventResponse;
import com.ruoyi.pt.service.CasesImportService;
import com.ruoyi.pt.service.EventsCopyService;
import com.ruoyi.pt.service.EventsService;
import com.ruoyi.pt.service.RemoteCallsService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wu
 * @since 2023/12/15 16:03
 */
@Service(value = "RemoteCallsService")
public class RemoteCallsServiceImpl implements RemoteCallsService {

    public static final Logger log = LoggerFactory.getLogger(RemoteCallsServiceImpl.class);

    @Value("${remote.url}")
    private String url;

    @Value("${remote.appKey}")
    private String appKey;

    @Value("${remote.appSecret}")
    private String appSecret;

    @Value("${remote.grantType}")
    private String grantType;

    @Resource
    private EventsService eventsService;

    @Resource
    private EventsCopyService eventsCopyService;

    @Resource
    private CasesImportService casesImportService;

    @Resource
    private RedisCache redisCache;

    /**
     * 获取填充信息后的map
     *
     * @return 填充信息后的map
     */
    private Map<String, Object> getMapValue() {
        Map<String, Object> map = new HashMap<>();
        map.put("appKey", appKey);
        map.put("appSecret", appSecret);
        map.put("grantType", grantType);
        return map;
    }

    /**
     * 获取公共accessToken
     *
     * @return accessToken
     */
    @Override
    public String getAccessToken() {
        Map<String, Object> form = getMapValue();
        String body = HttpRequest.post(url + "/auth/oauth/token").form(form).execute().body();
        RemoteCallsResponse res = JSONUtil.toBean(body, RemoteCallsResponse.class);
        log.info("==========获取token=========> " + res.getData().getAccessToken());
        return res.getData().getAccessToken();
    }

    /**
     * 事件进度推送接口
     *
     * @param allPush 是否全推送 1为推送所有 null为推送当天
     * @return 推送条数
     */
    @Override
    public int imports(Integer allPush) {
        // 推送条数
        int num = 0;
        String accessToken = this.getAccessToken();
        // 初始化MyBatis-Plus的分页对象 查询第1页，每页100条
        Page<Events> page = new Page<>(1, 100);
        // 创建查询条件
        QueryWrapper<Events> queryWrapper = new QueryWrapper<>();
        if (allPush != null && allPush == 1) {
            // 获取当天的日期
            //            LocalDate localDate = LocalDate.now();
            //            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            //            // 当天
            //            String eventTime = localDate.format(formatter);
            String eventTime;
            // 获取redis上次异常的时间
            Object obj = redisCache.getCacheObject("eventTime");
            if (BeanUtil.isNotEmpty(obj)) {
                eventTime = (String) obj;
                redisCache.deleteObject("eventTime");
            } else {
                //获取上个小时
                eventTime = DateUtil.offsetHour(DateUtil.date(), -1).toString("yyyy-MM-dd HH");
            }
            queryWrapper.and(wrapper -> wrapper.ge("eventTime", eventTime));
        }
        try {
            queryWrapper.and(wrapper -> wrapper.isNotNull("innerEventId").ne("innerEventId", "")).isNull("state");
            // 创建SimpleDateFormat对象，指定日期格式为yyyy-MM-dd HH:mm:ss
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            // 循环获取所有数据
            while (true) {
                IPage<Events> dataPage = eventsService.page(page, queryWrapper);
                List<Events> data = dataPage.getRecords();
                List<RemoteCallsDTO> dtos = BeanUtil.copyToList(data, RemoteCallsDTO.class);
                num += dtos.size();
                log.info("==========推送数据条数=========> " + dtos.size());
                dtos.forEach(x -> {
                    x.setContent(x.getTitle());
                    // 使用SimpleDateFormat对象将Date对象格式化为字符串
                    String firstWarnTime = sdf.format(x.getFirstWarnTimeDate());
                    String eventTime = sdf.format(x.getEventTimeDate());
                    x.setFirstWarnTime(firstWarnTime);
                    x.setEventTime(firstWarnTime);
                    x.setLastWarnTime(eventTime);
                    // 获取relationNums转成数组
                    JSONArray jsonArray = JSONUtil.parseArray(x.getRelationNums());
                    List<String> list = jsonArray.toList(String.class);
                    QueryWrapper<CasesImport> wrapper = new QueryWrapper<>();
                    wrapper.eq("CASE_SERIAL", list.get(0));
                    CasesImport casesImport = casesImportService.getOne(wrapper);
                    if (BeanUtil.isNotEmpty(casesImport)) {
                        // 如果是紧急事件content使用原始工单的content
                        if ("3".equals(x.getScenceType())) {
                            x.setContent(casesImport.getCaseContent());
                        } else {
                            x.setContent(x.getTitle());
                        }
                        // 如果类型是同人事件，上报⼈姓名和电话不取原始工单，直接从预警因素字段取，顿号分隔。
                        if ("2".equals(x.getScenceType())) {
                            String warnFactor = x.getWarnFactor();
                            String[] split = warnFactor.split("、");
                            if (split.length == 2) {
                                x.setEventCreatorTel(split[0]);
                                x.setEventCreator(split[1]);
                            } else {
                                x.setEventCreatorTel(split[0]);
                            }
                        } else {
                            x.setEventCreator(casesImport.getCreatorName());
                            x.setEventCreatorTel(casesImport.getCreatorTel());
                        }
                        x.setEventPlaceName(casesImport.getCaseAddress());
                        x.setEventCoordinate(casesImport.getCaseLnglat());
                    }
                });
                String jsonStr = JSONUtil.toJsonStr(dtos);
                HttpRequest.post(url + "/gateway/event/event/eventData/imports").header("Authorization", "Bearer " + accessToken).body(jsonStr).execute().body();
                // 判断是否还有下一页
                if (dataPage.getPages() > dataPage.getCurrent()) {
                    // 设置下一页的页码
                    page.setCurrent(page.getCurrent() + 1);
                } else {
                    log.info("==========推送完成=========");
                    // 已经获取完所有数据，结束循环
                    return num;
                }
            }
        } catch (HttpException e) {
            redisCache.setCacheObject("eventTime", DateUtil.offsetHour(DateUtil.date(), -1).toString("yyyy-MM-dd HH"));
            return 0;
        }
    }

    /**
     * 获取社会治理中心增量事件保存到events_import表
     *
     * @return 数据条数
     */
    @Override
    public int getEventDate() {
        Long id = casesImportService.findIdDesc();
        log.info("最新id是=============:" + id);
        if (id == null) {
            id = 0L;
        }
        return saveDate(id, 0);
    }

    /**
     * 紧急事件接口
     *
     * @param localDate 昨天日期字符串 yyyy-MM-dd
     */
    @Override
    public void urgentAlgorithm(String localDate) {
        Map<String, Object> body = new HashMap<>();
        body.put("data", localDate);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/urgent_algorithm").body(bodyStr).execute().body();
        log.info(localDate);
    }

    /**
     * 重点人员接口
     */
    @Override
    public void keypersonAlgorithm() {
        Map<String, Object> body = new HashMap<>();
        //data写死成3
        body.put("data", 3);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/keyperson_algorithm").body(bodyStr).execute().body();

    }

    /**
     * 群体事件接口
     */
    @Override
    public void groupAlgorithm() {
        Map<String, Object> body = new HashMap<>();
        body.put("data", 3);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/group_algorithm").body(bodyStr).execute().body();
    }

    /**
     * 同人事件接口
     */
    @Override
    public void onepersonAlgorithm() {
        Map<String, Object> body = new HashMap<>();
        body.put("data", 15);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/oneperson_algorithm").body(bodyStr).execute().body();
    }

    /**
     * 过滤审核不通过的事件列表
     *
     * @param lastHourStr  获取上个小时
     * @param localDateStr 获取当前时间
     */
    @Override
    public void noPassEventList(String lastHourStr, String localDateStr) {
        getTheData("getNoPassEventList", 1, lastHourStr, localDateStr);
    }

    /**
     * 过滤已办结的事件列表
     *
     * @param lastHourStr  获取上个小时
     * @param localDateStr 获取当前时间
     */
    @Override
    public void completionEventList(String lastHourStr, String localDateStr) {
        getTheData("getCompletionEventList", 2, lastHourStr, localDateStr);
    }

    /**
     * 根据事件id推送接口
     *
     * @param innerEventId 事件id
     */
    @Override
    public void pushByInnerEventId(String innerEventId) {
        String accessToken = this.getAccessToken();
        // 创建查询条件
        QueryWrapper<Events> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("innerEventId", innerEventId);
        // 创建SimpleDateFormat对象，指定日期格式为yyyy-MM-dd HH:mm:ss
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Events event = eventsService.getOne(queryWrapper);
        RemoteCallsDTO dtos = BeanUtil.copyProperties(event, RemoteCallsDTO.class);
        dtos.setContent(dtos.getTitle());
        // 使用SimpleDateFormat对象将Date对象格式化为字符串
        String firstWarnTime = sdf.format(dtos.getFirstWarnTimeDate());
        String eventTime = sdf.format(dtos.getEventTimeDate());
        dtos.setFirstWarnTime(firstWarnTime);
        dtos.setEventTime(firstWarnTime);
        dtos.setLastWarnTime(eventTime);
        // 获取relationNums转成数组
        JSONArray jsonArray = JSONUtil.parseArray(dtos.getRelationNums());
        List<String> list = jsonArray.toList(String.class);
        QueryWrapper<CasesImport> wrapper = new QueryWrapper<>();
        wrapper.eq("CASE_SERIAL", list.get(0));
        CasesImport casesImport = casesImportService.getOne(wrapper);
        if (BeanUtil.isNotEmpty(casesImport)) {
            // 如果是紧急事件content使用原始工单的content
            if ("3".equals(dtos.getScenceType())) {
                dtos.setContent(casesImport.getCaseContent());
            } else {
                dtos.setContent(dtos.getTitle());
            }
            // 如果类型是同人事件，上报⼈姓名和电话不取原始工单，直接从预警因素字段取，顿号分隔。
            if ("2".equals(dtos.getScenceType())) {
                String warnFactor = dtos.getWarnFactor();
                String[] split = warnFactor.split("、");
                if (split.length == 2) {
                    dtos.setEventCreatorTel(split[0]);
                    dtos.setEventCreator(split[1]);
                } else {
                    dtos.setEventCreatorTel(split[0]);
                }
            } else {
                dtos.setEventCreator(casesImport.getCreatorName());
                dtos.setEventCreatorTel(casesImport.getCreatorTel());
            }
            dtos.setEventPlaceName(casesImport.getCaseAddress());
            dtos.setEventCoordinate(casesImport.getCaseLnglat());
        }
        String jsonStr = JSONUtil.toJsonStr(dtos);
        HttpRequest.post(url + "/gateway/event/event/eventData/imports").header("Authorization", "Bearer " + accessToken).body(jsonStr).execute().body();
    }

    /**
     * 同人同诉接口
     */
    @Override
    public void samepersonComplaint() {
        Map<String, Object> body = new HashMap<>();
        body.put("data", 15);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/oneperson_algorithm").body(bodyStr).execute().body();
    }

    /**
     * 一人多诉接口
     */
    @Override
    public void onepersonSues() {
        Map<String, Object> body = new HashMap<>();
        body.put("data", 15);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/oneperson_muti_algorithm").body(bodyStr).execute().body();
    }

    /**
     * 获取社会治理中心增量事件保存到events_import表
     *
     * @param id  社会治理中心增量事件最大id
     * @param num 数据条数条数
     * @return 总获取过滤后的条数
     */
    public int saveDate(Long id, int num) {
        Map<String, Object> form = new HashMap<>();
        form.put("id", id);
        form.put("size", 5000);
        String body = HttpRequest.get(url + "/event/event/getEventAddListById").form(form).execute().body();
        RemoteEventResponse res = JSONUtil.toBean(body, RemoteEventResponse.class);
        Map<String, Object> map = JSONUtil.parseObj(res.getData());
        Object records = map.get("records");
        String recordsStr = JSONUtil.toJsonStr(records);
        List<CasesImport> list = JSONUtil.toList(recordsStr, CasesImport.class);
        log.info("=======================================");
        log.info("一共获取" + list.size() + "条数据");
        if (CollectionUtil.isNotEmpty(list)) {
            // 过滤数据
            List<CasesImport> filteredList = list.stream().filter(casesImport -> !casesImport.getEventExt().contains("12345便民-咨询")).collect(Collectors.toList());
            log.info("=======================================");
            log.info("过滤后剩下" + filteredList.size() + "条数据");
            num += filteredList.size();
            casesImportService.saveBatch(filteredList);
            id = Long.valueOf(list.get(list.size() - 1).getId());
            saveDate(id, num);
        }
        return num;
    }

    /**
     * 调取事件接口
     *
     * @param urlStr       请求地址
     * @param state        状态 1为未通过 2为已办结
     * @param lastHourStr  获取上个小时
     * @param localDateStr 获取当前时间
     */
    private void getTheData(String urlStr, Integer state, String lastHourStr, String localDateStr) {
        String accessToken = this.getAccessToken();
        Map<String, Object> form = new HashMap<>();
        form.put("size", 1000);
        if (StrUtil.isNotEmpty(lastHourStr) && StrUtil.isNotEmpty(localDateStr)) {
            form.put("startTime", lastHourStr);
            form.put("endTime", localDateStr);
        }
        int current = 1;
        // 循环获取所有数据
        while (true) {
            log.info("循环次数" + current + "================================");
            form.put("current", current);
            String body = HttpRequest.get(url + "/event/event/" + urlStr).header("Authorization", "Bearer " + accessToken).form(form).execute().body();
            RemoteEventResponse res = null;
            try {
                int i = body.indexOf("{");
                String bodyStr = body.substring(i);
                bodyStr = bodyStr.trim();
                res = JSONUtil.toBean(bodyStr, RemoteEventResponse.class);
            } catch (Exception e) {
                log.error("================================");
                log.error("接口地址是" + urlStr + "获取的数据为:");
                log.error(body);
                log.error("================================");
            }
            if (BeanUtil.isEmpty(res)) {
                return;
            }
            Map<String, Object> map = JSONUtil.parseObj(res.getData());
            Object records = map.get("records");
            String recordsStr = JSONUtil.toJsonStr(records);
            List<PassEventRequest> list = JSONUtil.toList(recordsStr, PassEventRequest.class);
            if (CollectionUtil.isNotEmpty(list)) {
                Events events = new Events();
                if (state == 1) {
                    List<String> notPassed = list.stream().filter(x -> x.getIfInstruct().equals("2")).map(PassEventRequest::getInnerEventId).collect(Collectors.toList());
                    log.info("IfInstruct=2的数据条数=====================" + notPassed.size() + "=========================");
                    if (CollectionUtil.isNotEmpty(notPassed)) {
                        QueryWrapper<Events> wrapper = new QueryWrapper<>();
                        wrapper.in("innerEventId", notPassed);
                        // 设置1
                        events.setState(state);
                        eventsService.update(events, wrapper);
                    }
                    List<String> disuse = list.stream().filter(x -> x.getIfInstruct().equals("4")).map(PassEventRequest::getInnerEventId).collect(Collectors.toList());
                    log.info("IfInstruct=4的数据条数=====================" + disuse.size() + "=========================");
                    if (CollectionUtil.isNotEmpty(disuse)) {
                        QueryWrapper<Events> wrapper = new QueryWrapper<>();
                        wrapper.in("innerEventId", disuse);
                        events.setState(3);
                        eventsService.update(events, wrapper);
                    }
                } else {
                    List<String> innerEventIdList = list.stream().map(PassEventRequest::getInnerEventId).collect(Collectors.toList());
                    QueryWrapper<Events> wrapper = new QueryWrapper<>();
                    wrapper.in("innerEventId", innerEventIdList);
                    // 设置2
                    events.setState(state);
                    eventsService.update(events, wrapper);
                }
                current++;
            } else {
                return;
            }
        }
    }
}
